% ph_morph.m
%
% Script called by 'Morph' menu item in photo editor.
% mode defines which block of code is executed. Valid modes are:
% 'init','new','add','delete','play','load','save','close'
%
% Nick Kingsbury, Cambridge University, March 2006.

switch mode
    
    case 'Init'  
        % Set up figure window for Morph commands
        figure(2);
        set(gcf,'position',[fig2pos  320  100]);
        set(gcf,'numbertitle','off','name','Morph Image');
        % Define buttons
        label = {'New','Add','Delete','Play','Load','Save',...
                'Select frame','Enter frame','Close'};
        for u = 1:6,
            btn(u) = uicontrol(gcf,'str',label{u},'pos',[50*u-40 60 50 20],...
                'call',sprintf('mode=label{%d}; ph_morph',u));
        end
        if exist('cp')~=1, cp = []; end % Initialise control-point matrix.
        cpstatus = uicontrol(gcf,'style','text','pos',[10 80 300 16],...
           'str',sprintf('%d Control Points in store.',size(cp,1)));
        
        % Other buttons: slider, edit box, and close.
        nframes = 10;
        slide = uicontrol(gcf,'style','slider','value',1,'pos',[10 10 120 20],...
            'sliderstep',[1 3]/(nframes-1),'call','mode=label{7}; ph_morph');
        uicontrol(gcf,'style','text','str',label{7},'pos',[10 30 120 16]);
        % set(slide,'call',['theta=(get(slide,''value'')-0.5)*360;']);
        edbox = uicontrol(gcf,'style','edit','string',nframes,...
            'pos',[150 10 60 20],'call','mode=label{8}; ph_morph');
        uicontrol(gcf,'style','text','str',label{8},'pos',[140 30 80 16]);
        
        closebtn = uicontrol(gcf,'str','Close','pos',[250 10 60 20],...
            'call','mode=''Close''; ph_morph');
        
        newsequence = 1;
        
    case 'New'
        cp = []; 
        xshift = 0; 
        newsequence = 1;
        yui = xui; 
        update_y; 
        mode='Get points';
        
    case 'Add'
        newsequence = 1;
        update_y; 
        mode='Get points';
        
    case 'Delete'
        newsequence = 1;
        % Remove most recent row of control points from cp,
        % recalculate result of morph, and display it.
        if size(cp,1) > 1, 
            cp = cp(1:(end-1),:); 
            [yui,xshift] = im_morph(xui,0,cp);
        else 
            cp = []; 
            xshift = 0;
            yui = xui;
        end
        set(cpstatus,'str',sprintf('%d Control Points in store.',size(cp,1)));
        update_y;
        figure(2);
        
    case 'Play'
        figure(1); 
        set(yhandle,'Erasemode','none');
        % If required, calculate new sequence of frames for movie.
        if newsequence,  
            yu = uint8(zeros([size(yui) nframes]));
            for k=1:nframes, 
                waitstr = sprintf('Morphing frame %d of %d',k,nframes);
                yui = im_morph(xui,((k-1)/(nframes-1))*xshift,[],waitstr);
                yu(:,:,:,k) = yui; 
                set(yhandle,'Cdata',yui); 
                drawnow;
            end;
            newsequence = 0;
            dk = 1; % direction = 'forward'
        end
        % Set up prompt and control button.
        set(promptbox,'str','Click button to morph back and forth');
        set(contrbutton,'str','Play again','call','mode=''Play''; ph_morph');
        % Play movie.
        for k=1:nframes, 
            yui = yu(:,:,:,k*dk + (nframes+1)*(1-dk)/2);
            set(yhandle,'Cdata',yui); 
            pause(0.05); 
        end; 
        dk = -dk; % swap direction for next 'play'.
        
    case 'Load'
        [Filename,PathName] = uigetfile({'*.mat','Morph cp files (*.mat)'},...
            'Open CP File');
        % Prompt user for a tif or jpeg filename.
        load([PathName Filename],'cp')
        set(cpstatus,'str',sprintf('%d Control Points in store.',size(cp,1)));
        [yui,xshift] = im_morph(xui,0,cp);
        update_y;
        newsequence = 1;
        figure(2);
        
    case 'Save'
        % Prompt user for an output filename.
        if ~exist('cpfile'), cpfile = 'morph_cp.mat'; end
        [FileName,PathName] = uiputfile(cpfile,'Save CP File');
        % Write yui to the file if Filename is valid.
        if FileName == 0,
            disp('Control-point file has not been saved.');
        else
            save([PathName FileName],'cp','infile');
        end
        
    case 'Select frame'
        k = round(get(slide,'value')*(nframes-1)) + 1;
        set(edbox,'string',sprintf('%d',k));
        yui = yu(:,:,:,k);
        set(yhandle,'Cdata',yui,'Erasemode','none'); 
        
    case 'Enter frame'
        k = max(min(round(sscanf(get(edbox,'string'),'%f')),nframes),1);
        set(slide,'value',(k-1)/(nframes-1));
        yui = yu(:,:,:,k);
        set(yhandle,'Cdata',yui,'Erasemode','none'); 
        
end


% The following modes can be called after the first group.
switch mode
    
    case 'Get points'
        set(contrbutton,'str','Terminate morph','call',' ');
        set(cpstatus,'str',sprintf('%d Control Points in store.',size(cp,1)));
        while 1,
            set(promptbox,'str','Place cross on morph start point'); 
            c0 = ginputnk(1,'crosshair');
            if isempty(c0) | any(c0<0), break; end;
            hold on; plot(c0(1),c0(2),'c+'); hold off;
            set(promptbox,'str','Place cross on morph end point'); 
            c1 = ginputnk(1,'cross');
            if isempty(c1) | any(c1<0), break; end;
            hold on; plot(c1(1),c1(2),'cx'); hold off;
            set(promptbox,'str','Place circle at radius from start point'); 
            c2 = ginputnk(1,'circle');
            if isempty(c2) | any(c2<0), break; end;
            rad = sqrt(sum((c2-c0).^2)); 
            cp = [cp; c0 c1 rad];
            [yui,xshift] = im_morph(xui,xshift,cp(end,:)); 
            update_y;
            set(cpstatus,'str',sprintf('%d Control Points in store.',size(cp,1)));
        end;
        newbefore=1; 
        showimages; 
        figure(2);  
        
    case 'Close'
        close(2); 
        newbefore = 1;
        showimages;
        
end
